/* -*- mode: c++ -*- */

/** @file

hippodraw::FigureEditor class interface and nested class implementation

Copyright (C) 2001-2006   The Board of Trustees of The Leland
Stanford Junior University.  All Rights Reserved.

$Id: FigureEditor.h,v 1.121 2007/03/21 18:22:06 pfkeb Exp $

*/

#ifndef FIGUREEDITOR_H
#define FIGUREEDITOR_H

#include "libqthippo.h"

#ifdef _MSC_VER
#include <msdevstudio/MSconfig.h>
#endif

#include <qglobal.h> // for version
#if QT_VERSION < 0x040000
#include <qcanvas.h>
#else
//Added by the Qt porting tool:
#include <QtCore/QCustomEvent>
#include <q3canvas.h>
#endif

#include <string>
#include <vector>

class QMouseEvent;
class QResizeEvent;

namespace hippodraw {

class PickTable;

/** This class implements additional QCanvasView functionality.   

    @attention This class knows about QCanvasItem objects.  Any
    methods that need to know about classes derived from QCanvasItem
    that are not part of Qt should be implemented in a derived class.

    The original source code was taken from
    $QTDIR/examples/canvas/canvas.h of version 3.0 of Qt.

    @bug @@@@@@ Mac OS X only, printer margins not showing in correct
    place on the Canvas, however printing is working fine.

    @request Move any attached text displays along with moved plotter
    display.

    @request @@@@@@ For plots using pixmap, do not draw plot when
    resizing until mouse is release.

    @todo Need ability to group and un-group items.

    @todo add page by adding another standard size page (either A4 or US
    Letter depending on preferences set somewhere.
    
    @author Kaustuv <kaustuv@stanford.edu>
    @author Paul F. Kunz <Paul_Kunz@slac.stanford.edu> 
    @author Sanket B. Malde <sanket@stanford.edu> 
    @author Engineers at <a
    href="http://www.trolltech.com"> Trolltech </a> who wrote
    examples/canvas
*/
#if QT_VERSION < 0x040000
class MDL_QTHIPPOPLOT_API FigureEditor : public QCanvasView 
#else
class MDL_QTHIPPOPLOT_API FigureEditor : public Q3CanvasView 
#endif
{

private:

  /** An Qt event class which wraps a QCanvasItem to be processed by the event.
      @author Paul F. Kunz <Paul_Kunz@slac.stanford.edu>
   */
class CanvasEvent : public QCustomEvent
{
private:
  /** The wrapped object.
   */
#if QT_VERSION < 0x040000
    const QCanvasItem * m_item;
#else
    const Q3CanvasItem * m_item;
#endif

public:

  /** Constructor taking the wrapped object as argument.
   */
#if QT_VERSION < 0x040000
    CanvasEvent ( const QCanvasItem * item )
#else
    CanvasEvent ( const Q3CanvasItem * item )
#endif
        : QCustomEvent ( 1000 ),
        m_item ( item ) 
    {}

  /** Returns the wrapped object to be used for the event processing.
   */
#if QT_VERSION < 0x040000
    const QCanvasItem * item () const
#else
    const Q3CanvasItem * item () const
#endif
  {
    return m_item;
  }
};

  /** The receiver of Qt custom events.
   */
  virtual void customEvent ( QCustomEvent * event );

protected: 

  /** The actual Page width in Postscript points. */
  int m_apage_w;

  /** The actual Page height in Postscript points. */
  int m_apage_h;

  /** The scale factor.  The factor by which drawing has been reduced
      so that a canvas page fits in the window.  This factor should be
      applied in inverse before printing.
  */
  double m_scale_factor;

  /** Set to @c true if zoom Mode is enabled
   */
  bool m_zoom_mode;

  /** The single QPrinter instance. */
  static QPrinter * s_printer;

  /** A flag to indicate if rectangle display usable printer boundary
      should be displayed or not.
  */
  bool m_printer_bounds;

  /** a flag set to @c true if the printer margins should be ignored
      when adding a plot to the canvas.
  */
  bool m_ignore_margin;

  /** Re-sizes the canvas if necessary so that the @a item is on the
      canvas.  Changes the height by one page size while keeping the
      width constant. */
#if QT_VERSION < 0x040000
  void resizeCanvasToFit ( QCanvasItem * item );
#else
  void resizeCanvasToFit ( Q3CanvasItem * item );
#endif

  /** Saves the printer setting with QSettings. */
  void savePrinterSettings ();

  /** Initializes the printer settings from the saved settings.  If
      setting have not been saved, set the default values. 
  */
  void initPrinter ();

  /** Adds page margin.  Adds an indication of the page's margin.  Only
      the contents inside the margin is printable. 
  */
  void addPageMargin ( int x, int y );

  /** Calculate the size of a page by using the printer metrics.  The
      size of a printable page is calculated on the basis that one
      pixel will become one Postscript point.  A QPaintDeviceMetrics
      object is created to find the actual and usable size of a page
      in millimeters which is then converted to Postscript Points.
  */
  void calcPrinterMetrics ( QPaintDevice * );

  /** Returns the largest Z value of all the items on the canvas.
      If the item is a text plotter, take z()-100 instead of z(). 
      This is to keep all text plotters on top of XyPlotters.
   */
  double maximumZ () const;

  /** Sets the unselected items to visible or not. 
   */
  void setUnselectedVisible ( bool yes );
  
  /** Move more than one items in the Canvas.
   */
  void mouseMoveMultiItem( QMouseEvent * e );


  /** Move a plotter with all its targetted text plotters.
   */
  void
#if QT_VERSION < 0X040000
  movePlotterWithText ( QCanvasItem * item, float dx, float dy );
#else 
  movePlotterWithText ( Q3CanvasItem * item, float dx, float dy );
#endif  
  

  /** Creates and returns a new QPixmap from the area defined on the canvas.
   */
  QPixmap * createPixmap ( const QRect & rectangle ) const;

  /** Creates and returns a QImage from the area defined on the
      canvas.
  */
  QImage createImage ( const QRect & rectange ) const;

protected:

  /** The usable Page width in Postscript points. */
  int m_upage_w;

  /** The usable Page height in Postscript points. */
  int m_upage_h;

  /** The list of view items on the canvas. This list is a subset of
      the one kept by the QCanvas object,That one doesn't seem to
      maintain the order that items were inserted while this one does.
      Also, only application items are added to this list, not page
      decorations.

      @attention All methods that add or remove items from the canvas
      should maintain this list.  That means QCanvasItem::setCanvas
      and QCanvasItem::~QCanvasItem.
   */
#if QT_VERSION < 0x040000
  std::vector < QCanvasItem * > m_items;
#else
  std::vector < Q3CanvasItem * > m_items;
#endif



  /** The view item which is clicked with right mouse button when 
      contentsMousePressEvent() happens.  When contentsMoveMouseEvent() 
      happens, get to this member, get the position of the cursor.
      When contentsReleaseMouseEvent() happens, reset the mouse cursor 
      to arrow.
  */
#if QT_VERSION < 0x040000
  QCanvasItem * m_rightItem;
#else
  Q3CanvasItem * m_rightItem;
#endif


  /** The view item which is preselected when left mouse button down.
      Useful only when multiple items are selected and one of them is
      pressed. Need to decide whether to select or to move all plots.
  */
#if QT_VERSION < 0x040000
  QCanvasItem * m_preSelected;
#else
  Q3CanvasItem * m_preSelected;
#endif


  /** The QSettings application key for searching for setting.  The
      initial value is "HippoDraw", but may be changed by Starkey.
  */
  static QString  s_app_key;

  /** The QSettings Windows registry key. The key is the same as used
      for by Trolltech for their Qt applications 
  */
  static QString s_registry;

  /** The canvas object.  Although one can always retrieve the canvas
      object by calling the canvas() member function, it is convenient
      to have this as a data member. */
#if QT_VERSION < 0x040000
  QCanvas * m_canvas;
#else
  Q3Canvas * m_canvas;
#endif

  /** The list of currently selected QCanvasItem objects. */
#if QT_VERSION < 0x040000
  std::vector < QCanvasItem * > m_selected_list;
#else
  std::vector < Q3CanvasItem * > m_selected_list;
#endif

  /** The starting point of a mouse drag. */
  QPoint m_moving_start;

   /** The starting point of a mouse drag.for zoom */
  QPoint m_zoom_start;

  /** The point of a mouse down, used when moving multiple plots. */
  QPoint m_mouse_down;

  /** The flag of mouse down or not, used when moving multiple plots. */
  bool m_isMouseDown;

  /** Possible locations where on a QCanvasItem the mouse cursor was
      when clicked. */
  enum Location { none, invalid,
		  upper_left,  upper_middle, upper_right,
		  middle_left, middle_right, 
		  lower_left,  lower_middle, lower_right };

  /** Location where the mouse cursor clicked the object. */
  Location m_whereClicked;

  /** Notifies Observer objects, if any, that something has changed. */
  virtual void notifyObservers () = 0;

  /** Scrolls the canvas, if needed, so the QCanvasItem @a item is
      visible. */
#if QT_VERSION < 0x040000
  void makeVisible ( const QCanvasItem * item );
#else
  void makeVisible ( const Q3CanvasItem * item );
#endif

  virtual void contentsMousePressEvent(QMouseEvent*);

  /** Handles details of mouse press event with shift key pressed. */
  virtual void controlMousePressEvent () = 0;

  /** Moves or re-sizes the selected QCanvasItem object.  The is moved
      if one of its knobs was selected, otherwise, it is re-sized. If
      @a aspect is non zero, then keeps its aspect ratio is kept while
      resizing. */

  void contentsMouseMoveEvent ( QMouseEvent *, double aspect );

  /** Handles details of mouse release, right button */
  void contentsMouseReleaseEvent (QMouseEvent *);


  /** Finds which knob on the selected item was clicked. */
  void whereClicked ( );

  /** Adds the @a item on top of the currently selected item. */
#if QT_VERSION < 0x040000
  void placeGraphOnSelected ( QCanvasRectangle * item );
#else
  void placeGraphOnSelected ( Q3CanvasRectangle * item );
#endif

  /** Sets the selected items to selection state @a state.  For each
      QCanvasItem in the selection list, set their selection state to
      @a state.  Does not alter the selection list. */
  void setSelectedItems ( bool state );

  /** Returns a QRect that enclosed the selected QCanvasItem objects. */
  QRect getSelectedBounds () const;

  /** De-selects all items, making the selection list empty. */
  void clearSelectedList ();

  /** Sets all QCanvasItems that collide with @a target to selected
      state.
  */
#if QT_VERSION < 0x040000
  void setCollidingSelected ( const QCanvasItem * target );
#else
  void setCollidingSelected ( const Q3CanvasItem * target );
#endif

  /** Save the selected objects as Pixmap image file @a filename. 
   */
  void saveSelectedAsPixmap ( const std::string & filename );

  /** Saves the area @a area as pixmap to file @a filename.
   */
  void saveAreaAsPixmap ( const QRect & area, const std::string & filename );

  /** Adds selected items to system clipboard.
   */
  void copySelectedToClipboard ();

  /** Prints the view to the @a printer. 
   */
  void print ( QPrinter * printer );

  /** Removes the item from list of items on the canvas.
   */
#if QT_VERSION < 0x040000
  void removeFromItemList ( QCanvasItem * item );
#else
  void removeFromItemList ( Q3CanvasItem * item );
#endif

#if QT_VERSION < 0x040000
  void addToItemList ( QCanvasItem * item );
#else
  void addToItemList ( Q3CanvasItem * item );
#endif

public:

  /** The constructor. */
#if QT_VERSION < 0x040000
  FigureEditor ( QCanvas *, QWidget * parent = 0, 
		 const char * name = 0, Qt::WFlags f = 0 );
#else
  FigureEditor ( Q3Canvas *, QWidget * parent = 0, 
		 const char * name = 0, Qt::WFlags f = 0 );
#endif

  /** Sets the QSettings application key. */
  void static setAppKey ( const std::string & );

  /** Sets showing the printer margins on or off.
   */
  void showPrinterMargins ( bool on );

  /** Set the default printer settings. */
  void setPrinterSettings ();

  /** Prints the view to the printer.  Presents the Print dialog to
      user and then prints the view to the selected printer. */
  virtual void print ();

    /** Prints the view to Postscript file @a filename.
   */
  virtual void print ( const std::string & filename );

  /** Re-sizes the QCanvas in response to parent window receiving
      QResizeEvent.  The QCanvas may grow in size, but is never made
      smaller by this method.*/
  void resizeEvent ( QResizeEvent * e );

  /** Adds a page to the canvas. */
  void addPage ();

  /** Returns the selected QCanvasItem. */
#if QT_VERSION < 0x040000
  QCanvasItem * selectedItem () const;
#else
  Q3CanvasItem * selectedItem () const;
#endif

  /** Set the selected QCanvasItem. This is called only when just one item
      is selected. */
#if QT_VERSION < 0x040000
  void setSelectedItem ( QCanvasItem * );
#else
  void setSelectedItem ( Q3CanvasItem * );
#endif


  /** Get the QCanvasItem clicked by right mouse button */
#if QT_VERSION < 0x040000
  QCanvasItem * getRightItem ();
#else
  Q3CanvasItem * getRightItem ();
#endif

  /** Sets selection flag on all QCanvasItem objects. */
  void setAllSelected ( bool flag = true );

  /** Sets the selected QCanvasItem to be locked in size and position. */
  void setLocked ( bool flag );


 /** Sets the zoom mode to @a flag */
  void setZoomMode ( bool flag );

  /** Returns the bool indicating whether zoom mode is enabled or not  */
  bool getZoomMode ( ) const;
  
  /** Removes all items from canvas and destroy them.  
   */
  void clear();

  /** Moves the QCanvasRectangle object to place on the canvas where
      it will not collide with existing QCanvasItem object and not be
      off the usable page area.
  */
#if QT_VERSION < 0x040000
  void placeGraph ( QCanvasRectangle * );
#else
  void placeGraph ( Q3CanvasRectangle * );
#endif

  /** Adds the item to the canvas.   Expands the canvas' size if necessary. */
#if QT_VERSION < 0x040000
  void add ( QCanvasItem * item );
#else
  void add ( Q3CanvasItem * item );
#endif

  /** Adds the item to the canvas.  This method differs from add in
      that it displaces the item by a fixed amount before adding. */
#if QT_VERSION < 0x040000
  void paste ( QCanvasItem * item );
#else
  void paste ( Q3CanvasItem * item );
#endif

  /** Removes the item from the canvas and destroys it.  @todo This
      function has a side effect of clearing the selection list,
      probably should not do that.
  */
#if QT_VERSION < 0x040000
  void remove ( QCanvasItem * item );
#else
  void remove ( Q3CanvasItem * item );
#endif

  /** Removes the item from the canvas but not destroys it. Used
      in grouping several items together.            @todo This
      function has a side effect of clearing the selection list,
      probably should not do that.
  */
#if QT_VERSION < 0x040000
  void hide ( QCanvasItem * item );
#else
  void hide ( Q3CanvasItem * item );
#endif

  /** Return the vector of selected items. */
#if QT_VERSION < 0x040000
  std::vector < QCanvasItem * > & getSelectedItems ();
#else
  std::vector < Q3CanvasItem * > & getSelectedItems ();
#endif

  /** Returns true if just one item is selected, else returns false. */
  bool isSingleItemSelected () const;

  /** Add an item to the selected list. */
#if QT_VERSION < 0x040000
  void addSelectedItem ( QCanvasItem * item );
#else
  void addSelectedItem ( Q3CanvasItem * item );
#endif

  /** Removes an item from the selection list.
   */
#if QT_VERSION < 0x040000
  void removeSelectedItem ( QCanvasItem * item );
#else
  void removeSelectedItem ( Q3CanvasItem * item );
#endif

  /** Set the selected flags. */
  void setSelectedFlags ();

  /** Scales the size of the images by @a factor.
      The factor might be modified to ensure we see
      at least a page in single glance of the canvas.
   */
  void scaleBy ( double factor );

  /** Enlarges the view of the canvas. */
  void viewZoomIn();

  /** Reduces the view of the canvas. */
  void viewZoomOut();

  /** Returns the view to normal size. 
   */
  void viewZoomReset ();

  /** Show the pick table. */
  virtual void viewShowPickTable() = 0;

  /** Scrolls the canvas, if needed, so the QCanvasItem @a item is
      visible. */
#if QT_VERSION < 0x040000
  void ensureVisible ( const QCanvasItem * item );
#else
  void ensureVisible ( const Q3CanvasItem * item );
#endif

  /** Returns a reference the application's QSettings key.
   */
  const QString & getAppKey () const;

  /** Returns a reference to the application's registry
   */
  const QString & getRegistry () const;

  /** Grabs the event when mouse cursor enters the widget.
   */
  void enterEvent ( QEvent * );

  /** Grabs the event when mouse cursor leaves the widget.
   */
  void leaveEvent ( QEvent * );


};

} // namespace hippodraw

#endif // FIGUREEDITOR_H
